/*
 * RCS Info
 * $Id: adms.css 990 2008-04-29 15:08:06Z r29173 $
 *
 * Log
 * $Log$
 * Revision 1.13  2006/05/29 07:53:51  r29173
 * improved display of stylesheet
 *
 * Revision 1.12  2005/11/30 13:57:21  r29173
 * improved css definition of button
 *
 * Revision 1.11  2005/11/22 09:29:47  r29173
 * added element comment
 *
 * Revision 1.10  2005/11/18 10:04:37  r29173
 * added span.comment
 *
 * Revision 1.9  2005/04/14 13:31:19  r29173
 * improved documentation
 *
 * Revision 1.8  2005/04/14 10:14:54  r29173
 * improved documentation
 *
 * Revision 1.7  2005/03/31 18:11:58  r29173
 * migration to adms-2.x.x:converted to adms2 syntax
 *
 * Revision 1.6  2005/03/17 11:46:04  r29173
 * migration to adms-2.x.x:cosmetics
 *
 * Revision 1.5  2005/03/17 08:26:28  r29173
 * migration to adms-2.x.x:move all href from absolute to relative values
 *
 * Revision 1.4  2005/03/16 17:37:47  r29173
 * migration to adms-2.x.x:cosmetics
 *
 * Revision 1.3  2005/03/16 17:31:24  r29173
 * migration to adms-2.x.x:small cosmetic
 *
 * Revision 1.2  2005/03/16 16:28:25  r29173
 * migration to adms-2.x.x:start xpath coding; add documentation
 *
 * Revision 1.1  2005/03/14 17:34:20  r29173
 * migration to adms-2.x.x:start xpath coding; add documentation
 *
*/

button
{
  border-color: gray;
  border-width: 1pt;
  font-size:8pt;
  color:maroon;
}
button.semanticHidden
{
  display:none;
}
button.semanticVisible
{
  display:inline;
  border-color: gray;
  border-width: 1pt;
  font-size:8pt;
  color:maroon;
}
body
{
  font-size:10pt;
  background-color:whitesmoke;
}
h1.title
{
  text-align:center;
  color:maroon;
  font-size:12pt;
}
h1
{
  color:maroon;
  font-size:12pt;
}
h2
{
  color:maroon;
  font-size:10pt;
  margin-left:10px;
}
span.comment
{
  color:maroon;
  font-size:10pt;
  margin-left:0px;
}
/* logos */
span.freescale
{
  color:red;
  font-size:10pt;
  margin-left:15px;
}
span.sourceforge
{
  color:red;
  font-size:10pt;
  margin-left:15px;
}

/* freegrammar grammar */
ul.freegrammar_productionset
{
  list-style-type:disc;
}
ul.freegrammar_productionset_production
{
  list-style-type:square;
}
li.freegrammar_productionset
{
  color:black;
}
li.productionset_production
{
  color:red;
}
span.production_constant
{
  font-weight:bold;
  color:black;
}
span.production_regexp
{
  font-weight:bold;
  color:magenta;
}
span.production_productionset
{
  font-weight:bold;
  color:blue;
}
span.admst_transform
{
  color:maroon;
  font-size:10pt;
}
span.admst_attribute
{
  color:maroon;
  font-size:10pt;
}
body
{
  font-size:10pt;
}
hr
{
  color:navy;
}
p
{
  font-size:10pt;
  margin-left:15px;
}
table
{
  font-size:10pt;
}
td.rowspan
{
  vertical-align:top;
}
td.notapplicable
{
  color:gray;
  font-size:10pt;
}
pre
{
  text-align:left;
  color:blue;
  font-size:8pt;
  margin-left:20px;
}
span.admst_obsolete
{
  color:blue;
}
span.info
{
  color:gray;
  font-style:italic;
  margin-left:5px;
}
span.shell_code, pre.admst_code, span.admst_transform
{
  border-style:groove;
  border-width:thin;
  background-color:white;
  text-align:left;
  color:black;
  font-size:8pt;
  margin-left:20px;
}
a:link
{
  color:maroon;
  text-decoration:none;
}
a:visited
{
  color:maroon;
  text-decoration:none;
}
a:hover
{
  color:black;
}
a:active
{
  color:blue;
}
.trigger
{
  cursor:pointer;
  display:block;
}
.branch
{
  display:none;
  margin-left:16px;
}
